Imports Syncfusion.Windows.Forms.Grid
Imports System.Runtime.Serialization

Public Class GridTextCellModel
    Inherits GridOriginalTextBoxCellModel

    Protected Sub New(ByVal info As SerializationInfo, ByVal context As StreamingContext)
        MyBase.New(info, context)
    End Sub

    Public Sub New(ByVal grid As GridModel)
        MyBase.New(grid)
    End Sub

    Public Overrides Function CreateRenderer(ByVal control As GridControlBase) As GridCellRendererBase
        Return New GridTextCellRenderer(control, Me)
    End Function
End Class

Public Class GridTextCellRenderer
    Inherits GridOriginalTextBoxCellRenderer

    Public Sub New(ByVal grid As GridControlBase, ByVal cellModel As GridOriginalTextBoxCellModel)
        MyBase.New(grid, cellModel)
    End Sub

    Protected Overrides Sub OnKeyDown(ByVal e As System.Windows.Forms.KeyEventArgs)
        Select Case e.KeyCode
            Case Keys.Right
                If e.Control Then
                    If Not e.Shift Then
                        Me.TextBox.SelectionStart = FindNextWordBreak(Me.TextBox.Text, Me.TextBox.SelectionStart)
                        Me.TextBox.SelectionLength = 0
                    Else
                        Me.TextBox.SelectionLength = FindNextWordBreak(Me.TextBox.Text, Me.TextBox.SelectionStart + Me.TextBox.SelectionLength) - Me.TextBox.SelectionStart
                    End If
                    e.Handled = True
                End If
            Case Keys.Left
                If e.Control Then
                    If Not e.Shift Then
                        Me.TextBox.SelectionStart = FindPreviousWordBreak(Me.TextBox.Text, Me.TextBox.SelectionStart)
                        Me.TextBox.SelectionLength = 0
                    Else
                        Dim SelStart As Integer = Me.TextBox.SelectionStart
                        Dim SelLength As Integer = Me.TextBox.SelectionLength
                        Dim WordBreak As Integer = FindPreviousWordBreak(Me.TextBox.Text, Me.TextBox.SelectionStart + Me.TextBox.SelectionLength)

                        If WordBreak < SelStart Then
                            Me.TextBox.SelectionStart = WordBreak
                            Me.TextBox.SelectionLength = SelStart + SelLength - WordBreak
                            Me.TextBox.SelectionStart = WordBreak
                        Else
                            Me.TextBox.SelectionLength = WordBreak - SelStart
                        End If
                    End If
                    e.Handled = True
                End If
            Case Else
                MyBase.OnKeyDown(e)
        End Select
    End Sub


    Friend Function FindNextWordBreak(ByVal Text As String, ByVal Start As Integer) As Integer
        Dim WordBreaks() As Char = New Char() {CType(",", Char), CType(".", Char), CType(";", Char), CType(":", Char), CType("-", Char), CType("!", Char), CType("""", Char), CType("", Char), CType("$", Char), CType("%", Char), CType("&", Char), CType("/", Char), CType("(", Char), CType(")", Char), CType("=", Char), CType("?", Char), CType("\", Char), CType("{", Char), CType("[", Char), CType("]", Char), CType("}", Char), CType("+", Char), CType("*", Char), CType("~", Char), CType("#", Char), CType("<", Char), CType(">", Char), CType("|", Char), CType(" ", Char)}
        Dim PosWordBreak As Integer = Start

        PosWordBreak = Text.IndexOfAny(WordBreaks, PosWordBreak)
        If PosWordBreak = -1 Then Return Text.Length
        While Text.Substring(PosWordBreak - 1, 1).IndexOfAny(WordBreaks) > -1
            PosWordBreak = Text.IndexOfAny(WordBreaks, PosWordBreak + 1)
            If PosWordBreak = -1 Then Return Text.Length
        End While

        Return PosWordBreak + 1
    End Function

    Friend Function FindPreviousWordBreak(ByVal Text As String, ByVal Start As Integer) As Integer
        Return Text.Length - FindNextWordBreak(ReverseText(Text), Text.Length - Start)
    End Function

    Friend Function ReverseText(ByVal Text As String) As String
        Dim Result As String = ""
        Dim I As Integer

        For I = Text.Length - 1 To 0 Step -1
            Result += Text.Substring(I, 1)
        Next

        Return Result
    End Function
End Class
